@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,8 +23,10 @@
 Show diagnostic information
 """
 
+from __future__ import absolute_import, print_function
+
+import bleachbit
 
-import Common
 import locale
 import os
 import platform
@@ -36,17 +38,20 @@ if 'nt' == os.name:
 
 def diagnostic_info():
     """Return diagnostic information as a string"""
-    s = "BleachBit version %s" % Common.APP_VERSION
+    s = "BleachBit version %s" % bleachbit.APP_VERSION
     try:
         import gtk
-        s += '\nGTK version %s' % '.'.join([str(x) for x in gtk.gtk_version])
-    except:
-        pass
-    s += "\nlocal_cleaners_dir = %s" % Common.local_cleaners_dir
-    s += "\nlocale_dir = %s" % Common.locale_dir
-    s += "\noptions_dir = %s" % Common.options_dir
-    s += "\npersonal_cleaners_dir = %s" % Common.personal_cleaners_dir
-    s += "\nsystem_cleaners_dir = %s" % Common.system_cleaners_dir
+        s += '\nGTK+ version %s' % '.'.join([str(x) for x in gtk.gtk_version])
+    except ImportError:
+        s += '\nGTK+ not detected'
+    import sqlite3
+    s += "\nSQLite version %s" % sqlite3.sqlite_version
+    s += "\nlocal_cleaners_dir = %s" % bleachbit.local_cleaners_dir
+    s += "\nlocale_dir = %s" % bleachbit.locale_dir
+    s += "\noptions_dir = %s" % bleachbit.options_dir.decode(bleachbit.FSE)
+    s += "\npersonal_cleaners_dir = %s" % bleachbit.personal_cleaners_dir.decode(
+        bleachbit.FSE)
+    s += "\nsystem_cleaners_dir = %s" % bleachbit.system_cleaners_dir
     s += "\nlocale.getdefaultlocale = %s" % str(locale.getdefaultlocale())
     if 'posix' == os.name:
         envs = ('DESKTOP_SESSION', 'LOGNAME', 'USER', 'SUDO_UID')
@@ -54,14 +59,33 @@ def diagnostic_info():
         envs = ('APPDATA', 'LocalAppData', 'LocalAppDataLow', 'Music',
                 'USERPROFILE', 'ProgramFiles', 'ProgramW6432', 'TMP')
     for env in envs:
-        s += "\nos.getenv('%s') = %s" % (env, os.getenv(env))
-    s += "\nos.path.expanduser('~') = %s" % os.path.expanduser('~')
+        if os.getenv(env):
+            s += "\nos.getenv('%s') = %s" % (env,
+                                             os.getenv(env).decode(bleachbit.FSE))
+        else:
+            s += "\nos.getenv('%s') = %s" % (env, os.getenv(env))
+    s += "\nos.path.expanduser('~') = %s" % bleachbit.expanduser(
+        '~').decode(bleachbit.FSE)
     if sys.platform.startswith('linux'):
         if hasattr(platform, 'linux_distribution'):
             s += "\nplatform.linux_distribution() = %s" % str(
                 platform.linux_distribution())
         else:
             s += "\nplatform.dist() = %s" % str(platform.dist())
+
+    # Mac Version Name - Dictionary
+    macosx_dict = {'5': 'Leopard', '6': 'Snow Leopard', '7': 'Lion', '8': 'Mountain Lion',
+                   '9': 'Mavericks', '10': 'Yosemite', '11': 'El Capitan', '12': 'Sierra'}
+
+    if sys.platform.startswith('darwin'):
+        if hasattr(platform, 'mac_ver'):
+            for key in macosx_dict:
+                if (platform.mac_ver()[0].split('.')[1] == key):
+                    s += "\nplatform.mac_ver() = %s" % str(
+                        platform.mac_ver()[0] + " (" + macosx_dict[key] + ")")
+        else:
+            s += "\nplatform.dist() = %s" % str(platform.dist())
+
     if 'nt' == os.name:
         s += "\nplatform.win32_ver[1]() = %s" % platform.win32_ver()[1]
     s += "\nplatform.platform = %s" % platform.platform()
