@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,7 +22,10 @@
 """
 File-related utilities
 """
+from __future__ import absolute_import, print_function
 
+import bleachbit
+from bleachbit import expanduser
 
 import atexit
 import codecs
@@ -36,17 +39,51 @@ import re
 import stat
 import string
 import sys
+import subprocess
 import tempfile
 import time
-import ConfigParser
-import Common
+
+logger = logging.getLogger(__name__)
 
 if 'nt' == os.name:
-    import pywintypes
+    from pywintypes import error as pywinerror
     import win32file
 
 if 'posix' == os.name:
-    from General import WindowsError
+    from bleachbit.General import WindowsError
+    pywinerror = WindowsError
+
+
+
+def open_files_linux():
+    return glob.iglob("/proc/*/fd/*")
+
+
+def open_files_lsof(run_lsof=None):
+    if run_lsof is None:
+        def run_lsof():
+            subprocess.check_output(["lsof", "-Fn", "-n"])
+    for f in run_lsof().split("\n"):
+        if f.startswith("n/"):
+            yield f[1:]  # Drop lsof's "n"
+
+
+def open_files():
+    if sys.platform.startswith('linux'):
+        files = open_files_linux()
+    elif 'darwin' == sys.platform:
+        files = open_files_lsof()
+    else:
+        raise RuntimeError('unsupported platform for open_files()')
+    for filename in files:
+        try:
+            target = os.path.realpath(filename)
+        except TypeError:
+            # happens, for example, when link points to
+            # '/etc/password\x00 (deleted)'
+            continue
+        else:
+            yield target
 
 
 class OpenFiles:
@@ -67,37 +104,31 @@ class OpenFiles:
         """Update cache"""
         self.last_scan_time = time.time()
         self.files = []
-        for filename in glob.iglob("/proc/*/fd/*"):
-            try:
-                target = os.path.realpath(filename)
-            except TypeError:
-                # happens, for example, when link points to
-                # '/etc/password\x00 (deleted)'
-                continue
-            if self.file_qualifies(target):
-                self.files.append(target)
+        for filename in open_files():
+            if self.file_qualifies(filename):
+                self.files.append(filename)
 
     def is_open(self, filename):
         """Return boolean whether filename is open by running process"""
-        if None == self.last_scan_time or (time.time() -
-                                           self.last_scan_time) > 10:
+        if self.last_scan_time is None or (time.time() - self.last_scan_time) > 10:
             self.scan()
-        return filename in self.files
+        return os.path.realpath(filename) in self.files
 
 
 def __random_string(length):
     """Return random alphanumeric characters of given length"""
     return ''.join(random.choice(string.ascii_letters + '0123456789_.-')
-                   for i in xrange(length))
+                   for i in range(length))
 
 
 def bytes_to_human(bytes_i):
+    # type: (int) -> str
     """Display a file size in human terms (megabytes, etc.) using preferred standard (SI or IEC)"""
 
     if bytes_i < 0:
         return '-' + bytes_to_human(-bytes_i)
 
-    from Options import options
+    from bleachbit.Options import options
     if options.get('units_iec'):
         prefixes = ['', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi']
         base = 1024.0
@@ -146,14 +177,14 @@ def clean_ini(path, section, parameter):
     """Delete sections and parameters (aka option) in the file"""
 
     # read file to parser
-    config = ConfigParser.RawConfigParser()
+    config = bleachbit.RawConfigParser()
     fp = codecs.open(path, 'r', encoding='utf_8_sig')
     config.readfp(fp)
 
     # change file
     changed = False
     if config.has_section(section):
-        if None == parameter:
+        if parameter is None:
             changed = True
             config.remove_section(section)
         elif config.has_option(section, parameter):
@@ -162,7 +193,7 @@ def clean_ini(path, section, parameter):
 
     # write file
     if changed:
-        from Options import options
+        from bleachbit.Options import options
         fp.close()
         if options.get('shred'):
             delete(path, True)
@@ -172,10 +203,7 @@ def clean_ini(path, section, parameter):
 
 def clean_json(path, target):
     """Delete key in the JSON file"""
-    try:
-        import json
-    except:
-        import simplejson as json
+    import json
     changed = False
     targets = target.split('/')
 
@@ -203,7 +231,7 @@ def clean_json(path, target):
             break
 
     if changed:
-        from Options import options
+        from bleachbit.Options import options
         if options.get('shred'):
             delete(path, True)
         # write file
@@ -216,8 +244,7 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
        If shred is enabled as a function parameter or the BleachBit global
        parameter, the path will be shredded unless allow_shred = False.
     """
-    from Options import options
-    logger = logging.getLogger(__name__)
+    from bleachbit.Options import options
     is_special = False
     path = extended_path(path)
     if not os.path.lexists(path):
@@ -237,14 +264,14 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
             delpath = wipe_name(path)
         try:
             os.rmdir(delpath)
-        except OSError, e:
+        except OSError as e:
             # [Errno 39] Directory not empty
             # https://bugs.launchpad.net/bleachbit/+bug/1012930
             if errno.ENOTEMPTY == e.errno:
                 logger.info("directory is not empty: %s", path)
             else:
                 raise
-        except WindowsError, e:
+        except WindowsError as e:
             # WindowsError: [Error 145] The directory is not empty:
             # 'C:\\Documents and Settings\\username\\Local Settings\\Temp\\NAILogs'
             # Error 145 may happen if the files are scheduled for deletion
@@ -258,9 +285,16 @@ def delete(path, shred=False, ignore_missing=False, allow_shred=True):
         if allow_shred and (shred or options.get('shred')):
             try:
                 wipe_contents(path)
-            except IOError, e:
+            except pywinerror as e:
+                # 2 = The system cannot find the file specified.
+                # This can happen with a broken symlink
+                # https://github.com/bleachbit/bleachbit/issues/195
+                if 2 != e.winerror:
+                    raise
+                # If a broken symlink, try os.remove() below.
+            except IOError as e:
                 # permission denied (13) happens shredding MSIE 8 on Windows 7
-                logger.debug("IOError #%s shredding '%s'", e.errno, path)
+                logger.debug("IOError #%s shredding '%s'", e.errno, path, exc_info=True)
             # wipe name
             os.remove(wipe_name(path))
         else:
@@ -299,21 +333,27 @@ def execute_sqlite3(path, cmds):
     import sqlite3
     conn = sqlite3.connect(path)
     cursor = conn.cursor()
+
+    # overwrites deleted content with zeros
+    # https://www.sqlite.org/pragma.html#pragma_secure_delete
+    from bleachbit.Options import options
+    if options.get('shred'):
+        cursor.execute('PRAGMA secure_delete=ON')
+
     for cmd in cmds.split(';'):
         try:
             cursor.execute(cmd)
-        except sqlite3.DatabaseError, exc:
+        except sqlite3.DatabaseError as exc:
             raise sqlite3.DatabaseError(
-                '%s: %s' % (Common.decode_str(exc), path))
-        except sqlite3.OperationalError, exc:
-            logger = logging.getLogger(__name__)
+                '%s: %s' % (bleachbit.decode_str(exc), path))
+        except sqlite3.OperationalError as exc:
             if exc.message.find('no such function: ') >= 0:
                 # fixme: determine why randomblob and zeroblob are not
                 # available
-                logger.warning(exc.message)
+                logger.exception(exc.message)
             else:
                 raise sqlite3.OperationalError(
-                    '%s: %s' % (Common.decode_str(exc), path))
+                    '%s: %s' % (bleachbit.decode_str(exc), path))
     cursor.close()
     conn.commit()
     conn.close()
@@ -322,23 +362,14 @@ def execute_sqlite3(path, cmds):
 def expand_glob_join(pathname1, pathname2):
     """Join pathname1 and pathname1, expand pathname, glob, and return as list"""
     ret = []
-    pathname3 = os.path.expanduser(
-        expandvars(os.path.join(pathname1, pathname2)))
+    pathname3 = expanduser(bleachbit.expandvars(os.path.join(pathname1, pathname2)))
     for pathname4 in glob.iglob(pathname3):
         ret.append(pathname4)
     return ret
 
 
 def expandvars(path):
-    if (2, 5) == sys.version_info[0:2] and 'nt' == os.name:
-        # Python 2.5 should not change $foo when foo is unknown, but
-        # it actually strips it out.
-        import backport
-        expandvars = backport.expandvars
-        return backport.expandvars(path)
-    else:
-        expandvars = os.path.expandvars
-        return os.path.expandvars(path)
+    return bleachbit.expandvars(path)
 
 
 def extended_path(path):
@@ -352,11 +383,29 @@ def extended_path(path):
     return path
 
 
+def extended_path_undo(path):
+    """"""
+    if 'nt' == os.name:
+        if path.startswith(r'\\?\unc'):
+            return '\\' + path[7:]
+        if path.startswith(r'\\?'):
+            return path[4:]
+    return path
+
+
 def free_space(pathname):
     """Return free space in bytes"""
     if 'nt' == os.name:
-        _, _, free_bytes = win32file.GetDiskFreeSpaceEx(pathname)
-        return free_bytes
+        from bleachbit import Windows
+        if Windows.parse_windows_build() >= 6:
+            # This works better with UTF-8 paths.
+            import psutil
+            return psutil.disk_usage(pathname).free
+        else:
+            # This works better with Windows XP but not UTF-8.
+            # Deprecated.
+            _, _, free_bytes = win32file.GetDiskFreeSpaceEx(pathname)
+            return free_bytes
     mystat = os.statvfs(pathname)
     return mystat.f_bfree * mystat.f_bsize
 
@@ -367,7 +416,7 @@ def getsize(path):
     if 'posix' == os.name:
         try:
             __stat = os.lstat(path)
-        except OSError, e:
+        except OSError as e:
             # OSError: [Errno 13] Permission denied
             # can happen when a regular user is trying to find the size of /var/log/hp/tmp
             # where /var/log/hp is 0774 and /var/log/hp/tmp is 1774
@@ -381,7 +430,7 @@ def getsize(path):
         # Also, apply prefix to use extended-length paths to support longer
         # filenames.
         finddata = win32file.FindFilesW(extended_path(path))
-        if finddata == []:
+        if not finddata:
             # FindFilesW does not work for directories, so fall back to
             # getsize()
             return os.path.getsize(path)
@@ -417,22 +466,18 @@ def guess_overwrite_paths():
     # ~/.config makes it easy to find them and clean them.
     ret = []
     if 'posix' == os.name:
-        home = os.path.expanduser('~/.cache')
+        home = expanduser('~/.cache')
         if not os.path.exists(home):
-            home = os.path.expanduser("~")
+            home = expanduser("~")
         ret.append(home)
         if not same_partition(home, '/tmp/'):
             ret.append('/tmp')
     elif 'nt' == os.name:
-        localtmp = expandvars('$TMP')
-        logger = logging.getLogger(__name__)
+        localtmp = bleachbit.expandvars('$TMP')
         if not os.path.exists(localtmp):
             logger.warning('%TMP% does not exist: %s', localtmp)
             localtmp = None
-        else:
-            from win32file import GetLongPathName
-            localtmp = GetLongPathName(localtmp)
-        from Windows import get_fixed_drives
+        from bleachbit.Windows import get_fixed_drives
         for drive in get_fixed_drives():
             if localtmp and same_partition(localtmp, drive):
                 ret.append(localtmp)
@@ -447,20 +492,25 @@ def human_to_bytes(human, hformat='si'):
     """Convert a string like 10.2GB into bytes.  By
     default use SI standard (base 10).  The format of the
     GNU command 'du' (base 2) also supported."""
+
     if 'si' == hformat:
-        multiplier = {'B': 1, 'kB': 1000, 'MB': 1000 ** 2,
-                      'GB': 1000 ** 3, 'TB': 1000 ** 4}
-        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([kMGT]{0,1}B)$", human)
+        base = 1000
+        suffixes = 'kMGTE'
     elif 'du' == hformat:
-        multiplier = {'B': 1, 'KB': 1024, 'MB': 1024 ** 2,
-                      'GB': 1024 ** 3, 'TB': 1024 ** 4}
-        matches = re.findall("^([0-9]*)(\.[0-9]{1,2})?([KMGT]{0,1}B)$", human)
+        base = 1024
+        suffixes = 'KMGTE'
     else:
         raise ValueError("Invalid format: '%s'" % hformat)
-    if [] == matches or 2 > len(matches[0]):
-        raise ValueError(
-            "Invalid input for '%s' (hformat='%s')" % (human, hformat))
-    return int(float(matches[0][0] + matches[0][1]) * multiplier[matches[0][2]])
+    matches = re.match(r'^(\d+(?:\.\d+)?) ?([' + suffixes + ']?)B?$', human)
+    if matches is None:
+        raise ValueError("Invalid input for '%s' (hformat='%s')" % (human, hformat))
+    (amount, suffix) = matches.groups()
+
+    if '' == suffix:
+        exponent = 0
+    else:
+        exponent = suffixes.find(suffix) + 1
+    return int(float(amount) * base**exponent)
 
 
 def listdir(directory):
@@ -473,7 +523,7 @@ def listdir(directory):
             for pathname in listdir(dirname):
                 yield pathname
         return
-    dirname = os.path.expanduser(directory)
+    dirname = expanduser(directory)
     if not os.path.lexists(dirname):
         return
     for filename in os.listdir(dirname):
@@ -485,7 +535,7 @@ def same_partition(dir1, dir2):
     if 'nt' == os.name:
         try:
             return free_space(dir1) == free_space(dir2)
-        except pywintypes.error, e:
+        except pywinerror as e:
             if 5 == e.winerror:
                 # Microsoft Office 2010 Starter Edition has a virtual
                 # drive that gives access denied
@@ -505,16 +555,28 @@ def sync():
         import ctypes
         rc = ctypes.cdll.LoadLibrary('libc.so.6').sync()
         if 0 != rc:
-            logger = logging.getLogger(__name__)
-            logge.error('sync() returned code %d' % rc)
+            logger.error('sync() returned code %d', rc)
     if 'nt' == os.name:
         import ctypes
         ctypes.cdll.LoadLibrary('msvcrt.dll')._flushall()
 
 
-def whitelisted(path):
-    """Check whether this path is whitelisted"""
-    from Options import options
+def truncate_f(f):
+    """Truncate the file object"""
+    f.truncate(0)
+    f.flush()
+    os.fsync(f.fileno())
+
+
+def whitelisted_posix(path, check_realpath = True):
+    """Check whether this POSIX path is whitelisted"""
+    from bleachbit.Options import options
+    if check_realpath and os.path.islink(path):
+        # also check the link name
+        if whitelisted_posix(path, False):
+            return True
+        # resolve symlink
+        path = os.path.realpath(path)
     for pathname in options.get_whitelist_paths():
         if pathname[0] == 'file' and path == pathname[1]:
             return True
@@ -523,21 +585,32 @@ def whitelisted(path):
                 return True
             if path.startswith(pathname[1] + os.sep):
                 return True
-        if 'nt' == os.name:
-            # Windows is case insensitive
-            if pathname[0] == 'file' and path.lower() == pathname[1].lower():
-                return True
-            if pathname[0] == 'folder':
-                if path.lower() == pathname[1].lower():
-                    return True
-                if path.lower().startswith(pathname[1].lower() + os.sep):
-                    return True
-                # Simple drive letter like C:\ matches everything below
-                if len(pathname[1]) == 3 and path.lower().startswith(pathname[1].lower()):
-                    return True
     return False
 
 
+def whitelisted_windows(path):
+    """Check whether this Windows path is whitelisted"""
+    from bleachbit.Options import options
+    for pathname in options.get_whitelist_paths():
+        # Windows is case insensitive
+        if pathname[0] == 'file' and path.lower() == pathname[1].lower():
+            return True
+        if pathname[0] == 'folder':
+            if path.lower() == pathname[1].lower():
+                return True
+            if path.lower().startswith(pathname[1].lower() + os.sep):
+                return True
+            # Simple drive letter like C:\ matches everything below
+            if len(pathname[1]) == 3 and path.lower().startswith(pathname[1].lower()):
+                return True
+    return False
+
+if 'nt' == os.name:
+    whitelisted = whitelisted_windows
+else:
+    whitelisted = whitelisted_posix
+
+
 def wipe_contents(path, truncate=True):
     """Wipe files contents
 
@@ -546,31 +619,69 @@ def wipe_contents(path, truncate=True):
     shown that most of today's media can be effectively cleared
     by one overwrite"
     """
-    size = getsize(path)
-    try:
-        f = open(path, 'wb')
-    except IOError, e:
-        if e.errno == errno.EACCES:  # permission denied
-            os.chmod(path, 0200)  # user write only
+
+    def wipe_write():
+        size = getsize(path)
+        try:
+            f = open(path, 'wb')
+        except IOError as e:
+            if e.errno == errno.EACCES:  # permission denied
+                os.chmod(path, 0o200)  # user write only
+                f = open(path, 'wb')
+            else:
+                raise
+        blanks = chr(0) * 4096
+        while size > 0:
+            f.write(blanks)
+            size -= 4096
+        f.flush()  # flush to OS buffer
+        os.fsync(f.fileno())  # force write to disk
+        return f
+
+    if 'nt' == os.name:
+        from win32com.shell.shell import IsUserAnAdmin
+
+    if 'nt' == os.name and IsUserAnAdmin():
+        from bleachbit.WindowsWipe import file_wipe, UnsupportedFileSystemError
+        import warnings
+        from bleachbit import _
+        try:
+            file_wipe(path)
+        except pywinerror as e:
+            # 32=The process cannot access the file because it is being used by another process.
+            # 33=The process cannot access the file because another process has
+            # locked a portion of the file.
+            if not e.winerror in (32, 33):
+                # handle only locking errors
+                raise
+            # Try to truncate the file. This makes the behavior consistent
+            # with Linux and with Windows when IsUserAdmin=False.
+            try:
+                with open(path, 'wb') as f:
+                    truncate_f(f)
+            except IOError as e2:
+                if errno.EACCES == e2.errno:
+                    # Common when the file is locked
+                    # Errno 13 Permission Denied
+                    pass
+            # translate exception to mark file to deletion in Command.py
+            raise WindowsError(e.winerror, e.strerror)
+        except UnsupportedFileSystemError as e:
+            warnings.warn(
+                _('There was at least one file on a file system that does not support advanced overwriting.'), UserWarning)
             f = open(path, 'wb')
         else:
-            raise
-    blanks = chr(0) * 4096
-    while size > 0:
-        f.write(blanks)
-        size -= 4096
-    f.flush()  # flush to OS buffer
-    os.fsync(f.fileno())  # force write to disk
+            # The wipe succeed, so prepare to truncate.
+            f = open(path, 'wb')
+    else:
+        f = wipe_write()
     if truncate:
-        f.truncate(0)
-        f.flush()
-        os.fsync(f.fileno())
+        truncate_f(f)
     f.close()
 
 
 def wipe_name(pathname1):
     """Wipe the original filename and return the new pathname"""
-    logger = logging.getLogger(__name__)
     (head, _) = os.path.split(pathname1)
     # reference http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
     maxlen = 226
@@ -597,7 +708,7 @@ def wipe_name(pathname1):
             os.rename(pathname2, pathname3)
             break
         except:
-            i = i + 1
+            i += 1
             if i > 100:
                 logger.info('exhausted short rename: %s', pathname2)
                 pathname3 = pathname2
@@ -609,8 +720,6 @@ def wipe_path(pathname, idle=False):
     """Wipe the free space in the path
     This function uses an iterator to update the GUI."""
 
-    logger = logging.getLogger(__name__)
-
     def temporaryfile():
         # reference
         # http://en.wikipedia.org/wiki/Comparison_of_file_systems#Limits
@@ -618,17 +727,14 @@ def wipe_path(pathname, idle=False):
         f = None
         while True:
             try:
-                kwargs = {
-                    'dir': pathname, 'suffix': __random_string(maxlen)}
-                if sys.hexversion >= 0x02060000:
-                    kwargs['delete'] = False
-                f = tempfile.NamedTemporaryFile(**kwargs)
+                f = tempfile.NamedTemporaryFile(
+                    dir=pathname, suffix=__random_string(maxlen), delete=False)
                 # In case the application closes prematurely, make sure this
                 # file is deleted
                 atexit.register(
                     delete, f.name, allow_shred=False, ignore_missing=True)
                 break
-            except OSError, e:
+            except OSError as e:
                 if e.errno in (errno.ENAMETOOLONG, errno.ENOSPC, errno.ENOENT):
                     # ext3 on Linux 3.5 returns ENOSPC if the full path is greater than 264.
                     # Shrinking the size helps.
@@ -655,7 +761,7 @@ def wipe_path(pathname, idle=False):
         done_time = time.time() - start_time
         rate = done_bytes / (done_time + 0.0001)  # bytes per second
         remaining_seconds = int(remaining_bytes / (rate + 0.0001))
-        return (1, done_percent, remaining_seconds)
+        return 1, done_percent, remaining_seconds
 
     logger.debug("wipe_path('%s')", pathname)
     files = []
@@ -668,7 +774,7 @@ def wipe_path(pathname, idle=False):
         try:
             logger.debug('creating new, temporary file to wipe path')
             f = temporaryfile()
-        except OSError, e:
+        except OSError as e:
             # Linux gives errno 24
             # Windows gives errno 28 No space left on device
             if e.errno in (errno.EMFILE, errno.ENOSPC):
@@ -681,7 +787,7 @@ def wipe_path(pathname, idle=False):
         while True:
             try:
                 f.write(blanks)
-            except IOError, e:
+            except IOError as e:
                 if e.errno == errno.ENOSPC:
                     if len(blanks) > 1:
                         # Try writing smaller blocks
@@ -702,7 +808,7 @@ def wipe_path(pathname, idle=False):
             # IOError: [Errno 28] No space left on device
             # seen on Microsoft Windows XP SP3 with ~30GB free space but
             # not on another XP SP3 with 64MB free space
-            logger.info("info: exception on f.flush()")
+            logger.info("info: exception on f.flush()", exc_info=True)
         os.fsync(f.fileno())  # write to disk
         # Remember to delete
         files.append(f)
@@ -727,7 +833,7 @@ def wipe_path(pathname, idle=False):
                     stats.f_bsize * stats.f_bfree, stats.f_ffree)
     # truncate and close files
     for f in files:
-        f.truncate(0)
+        truncate_f(f)
 
         while True:
             try:
@@ -736,14 +842,12 @@ def wipe_path(pathname, idle=False):
                 # to do actually close (and therefore delete) a temporary file
                 f.close()
                 break
-            except IOError, e:
+            except IOError as e:
                 if e.errno == 0:
                     logger.debug('handled unknown error 0')
                     time.sleep(0.1)
         # explicitly delete
-        # Python 2.5.4 always deletes NamedTemporaryFile, so
-        # ignore missing in older Python here.
-        delete(f.name, ignore_missing=sys.hexversion < 0x02060000)
+        delete(f.name, ignore_missing=True)
 
 
 def vacuum_sqlite3(path):
