@@ -2,8 +2,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+from __future__ import absolute_import, print_function
+
+import bleachbit
+
+from bleachbit.Cleaner import backends, register_cleaners
+from bleachbit.GuiPreferences import PreferencesDialog
+from bleachbit.Options import options
+from bleachbit import _, _p, APP_NAME, appicon_path, portable_mode
+from bleachbit import Cleaner, FileUtilities
+from bleachbit import GuiBasic
 
 import logging
 import os
@@ -34,18 +44,10 @@ import gtk
 import gobject
 warnings.simplefilter('default')
 
-from Common import _, _p, APP_NAME, APP_VERSION, APP_URL, appicon_path, \
-    help_contents_url, license_filename, options_file, options_dir, \
-    online_update_notification_enabled, release_notes_url, portable_mode
-from Cleaner import backends, register_cleaners
-from GuiPreferences import PreferencesDialog
-from Options import options
-import Cleaner
-import FileUtilities
-import GuiBasic
-
 if 'nt' == os.name:
-    import Windows
+    from bleachbit import Windows
+
+logger = logging.getLogger(__name__)
 
 
 def threaded(func):
@@ -96,10 +98,8 @@ class TreeInfoModel:
             c_name = backends[key].get_name()
             c_id = backends[key].get_id()
             c_value = options.get_tree(c_id, None)
-            if not c_value and options.get('auto_hide') \
-                    and backends[key].auto_hide():
-                logger = logging.getLogger(__name__)
-                logger.debug("automatically hiding cleaner '%s'", (c_id))
+            if not c_value and options.get('auto_hide') and backends[key].auto_hide():
+                logger.debug("automatically hiding cleaner '%s'", c_id)
                 continue
             parent = self.tree_store.append(None, (c_name, c_value, c_id, ""))
             for (o_id, o_name) in backends[key].get_options():
@@ -360,7 +360,7 @@ class GUI:
         """Preview operations or run operations (delete files)"""
 
         assert(isinstance(really_delete, bool))
-        import Worker
+        from bleachbit import Worker
         self.start_time = None
         if None == operations:
             operations = {}
@@ -377,8 +377,7 @@ class GUI:
             self.textbuffer.set_text("")
             self.progressbar.show()
             self.worker = Worker.Worker(self, really_delete, operations)
-        except Exception, e:
-            logger = logging.getLogger(__name__)
+        except Exception:
             logger.exception('Error in Worker()')
         else:
             self.start_time = time.time()
@@ -402,13 +401,12 @@ class GUI:
 
         # notification for long-running process
         elapsed = (time.time() - self.start_time)
-        logger = logging.getLogger(__name__)
         logger.debug('elapsed time: %d seconds', elapsed)
         if elapsed < 10 or self.window.is_active():
             return
         try:
             import pynotify
-        except:
+        except ImportError:
             logger.debug('pynotify not available')
         else:
             if pynotify.init(APP_NAME):
@@ -426,10 +424,14 @@ class GUI:
                                           link: GuiBasic.open_url(link, self.window, False))
         dialog = gtk.AboutDialog()
         dialog.set_comments(_("Program to clean unnecessary files"))
-        dialog.set_copyright("Copyright (C) 2008-2016 Andrew Ziem")
+        dialog.set_copyright("Copyright (C) 2008-2018 Andrew Ziem")
         try:
-            dialog.set_license(open(license_filename).read())
-        except:
+            with open(bleachbit.license_filename) as f:
+                dialog.set_license(f.read())
+        except (IOError, TypeError):
+            # In case the license file cannot be read, there will be an
+            # IOError. In case the license file does not exist, the filename
+            # will be none, causing a TypeError.
             dialog.set_license(
                 _("GNU General Public License version 3 or later.\nSee http://www.gnu.org/licenses/gpl-3.0.txt"))
         dialog.set_name(APP_NAME)
@@ -438,8 +440,8 @@ class GUI:
         # typed in Launchpad. This is a special string shown
         # in the 'About' box.
         dialog.set_translator_credits(_("translator-credits"))
-        dialog.set_version(APP_VERSION)
-        dialog.set_website(APP_URL)
+        dialog.set_version(bleachbit.APP_VERSION)
+        dialog.set_website(bleachbit.APP_URL)
         dialog.set_transient_for(self.window)
         if appicon_path and os.path.exists(appicon_path):
             icon = gtk.gdk.pixbuf_new_from_file(appicon_path)
@@ -452,7 +454,7 @@ class GUI:
         dialog = gtk.Dialog(_("System information"), parent)
         dialog.resize(600, 400)
         txtbuffer = gtk.TextBuffer()
-        import Diagnostic
+        from bleachbit import Diagnostic
         txt = Diagnostic.diagnostic_info()
         txtbuffer.set_text(txt)
         textview = gtk.TextView(txtbuffer)
@@ -561,23 +563,23 @@ class GUI:
         """Shred settings (for privacy reasons) and quit"""
         # build a list of paths to delete
         paths = []
-        if portable_mode:
+        if 'nt' == os.name and portable_mode:
             # in portable mode on Windows, the options directory includes
             # executables
-            paths.append(options_file)
+            paths.append(bleachbit.options_file)
         else:
-            paths.append(options_dir)
+            paths.append(bleachbit.options_dir)
 
         # prompt the user to confirm
         if not self.shred_paths(paths):
-            logger = logging.getLogger(__name__)
             logger.debug('user aborted shred')
             # aborted
             return
 
         # in portable mode, rebuild a minimal bleachbit.ini
-        if portable_mode:
-            open(options_file, 'w').write('[Portable]\n')
+        if 'nt' == os.name and portable_mode:
+            with open(bleachbit.options_file, 'w') as f:
+                f.write('[Portable]\n')
 
         # Quit the application through the idle loop to allow the worker
         # to delete the files.  Use the lowest priority because the worker
@@ -618,7 +620,6 @@ class GUI:
         model = treeview.get_model()
         option_id = model[path][2]
         cleaner_id = model[path[0]][2]
-        cleaner_name = model[path[0]][0]
         # make a menu
         menu = gtk.Menu()
         # TRANSLATORS: this is the context menu
@@ -638,6 +639,44 @@ class GUI:
         menu.popup(None, None, None, event.button, event.time)
         return True
 
+    def cb_drag_motion(self, widget, drag_context, x, y, time):
+        drag_context.drag_status(gtk.gdk.ACTION_MOVE, time)
+        return True
+
+    def cb_drag_data_received(self, widget, context, x, y, selection, target_type, time):
+        if target_type == self.TARGET_TYPE_TEXT:
+            import urlparse
+            import urllib
+            assert(type(selection.data) is str)
+            file_urls = selection.data.split("\n")
+            file_paths = []
+            for file_url in file_urls:
+                # strip needed to remove "\r"
+                file_url = file_url.strip()
+                parsed_url = urlparse.urlparse(file_url)
+                if parsed_url.scheme == "file":
+                    file_path = urllib.url2pathname(parsed_url.path)
+                    file_path_unicode = file_path.decode("utf-8")
+                    file_paths.append(file_path_unicode)
+
+            self.shred_paths(file_paths)
+
+    def setup_drag_n_drop_widget(self, widget):
+        targets = [("text/uri-list", 0, self.TARGET_TYPE_TEXT)]
+
+        widget.connect("drag_motion", self.cb_drag_motion)
+        widget.connect("drag_data_received", self.cb_drag_data_received)
+
+        widget.drag_dest_set(gtk.DEST_DEFAULT_DROP,
+                             targets, gtk.gdk.ACTION_MOVE)
+
+    def setup_drag_n_drop(self):
+        self.TARGET_TYPE_TEXT = 80
+
+        self.setup_drag_n_drop_widget(self.window)
+        # need to update textview drag&drop settings too
+        self.setup_drag_n_drop_widget(self.textview)
+
     def update_progress_bar(self, status):
         """Callback to update the progress bar with number or text"""
         if type(status) is float:
@@ -658,7 +697,7 @@ class GUI:
         __iter = model.get_iter_root()
         while __iter:
             if model[__iter][2] == option:
-                if (option_id == -1):
+                if option_id == -1:
                     model[__iter][3] = text
                 else:
                     child = model.iter_children(__iter)
@@ -707,10 +746,10 @@ class GUI:
             ('Edit', None, _("_Edit")),
             ('HelpContents', gtk.STOCK_HELP, _('Help Contents'), 'F1', None,
              lambda link: GuiBasic.open_url(
-             help_contents_url, self.window)),
+             bleachbit.help_contents_url, self.window)),
             ('ReleaseNotes', gtk.STOCK_INFO, _('_Release Notes'), None, None,
              lambda link: GuiBasic.open_url(
-             release_notes_url, self.window)),
+             bleachbit.release_notes_url, self.window)),
             ('SystemInformation', None, _('_System Information'), None,
              None, lambda foo: self.diagnostic_dialog(self.window)),
             ('About', gtk.STOCK_ABOUT, _(
@@ -829,6 +868,9 @@ class GUI:
         self.status_bar = gtk.Statusbar()
         vbox.pack_start(self.status_bar, False)
 
+        # setup drag&drop
+        self.setup_drag_n_drop()
+
         # done
         self.window.show_all()
         self.progressbar.hide()
@@ -837,7 +879,7 @@ class GUI:
     @threaded
     def check_online_updates(self):
         """Check for software updates in background"""
-        import Update
+        from bleachbit import Update
         try:
             updates = Update.check_updates(options.get('check_beta'),
                                            options.get('update_winapp2'),
@@ -846,17 +888,17 @@ class GUI:
             if updates:
                 gobject.idle_add(
                     lambda: Update.update_dialog(self.window, updates))
-        except Exception, e:
-            logger = logging.getLogger(__name__)
+        except Exception:
             logger.exception(_("Error when checking for updates: "))
 
-    def __init__(self, uac=True, shred_paths=None):
+    def __init__(self, uac=True, shred_paths=None, exit=False):
         if uac and 'nt' == os.name and Windows.elevate_privileges():
             # privileges escalated in other process
             sys.exit(0)
-        import RecognizeCleanerML
-        RecognizeCleanerML.RecognizeCleanerML()
-        register_cleaners()
+        if not exit:
+            from bleachbit import RecognizeCleanerML
+            RecognizeCleanerML.RecognizeCleanerML()
+            register_cleaners()
         self.create_window()
         gobject.threads_init()
 
@@ -868,7 +910,7 @@ class GUI:
             # On Microsoft Windows this avoids py2exe redirecting stderr to
             # bleachbit.exe.log.
             # sys.frozen = console_exe means the console is shown
-            from Common import logger_sh
+            from bleachbit import logger_sh
             bb_logger.removeHandler(logger_sh)
         if shred_paths:
             self.shred_paths(shred_paths)
@@ -877,21 +919,29 @@ class GUI:
             pref = PreferencesDialog(self.window, self.cb_refresh_operations)
             pref.run()
             options.set('first_start', False)
-        if online_update_notification_enabled and options.get("check_online_updates"):
+        if bleachbit.online_update_notification_enabled and options.get("check_online_updates"):
             self.check_online_updates()
         if 'nt' == os.name:
             # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
             # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
-            # http://www.bleachbit.org/forum/074-fails-errors
+            # https://www.bleachbit.org/forum/074-fails-errors
             try:
                 import sqlite3
-            except ImportError, e:
-                logger = logging.getLogger(__name__)
-                logger.exception(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."))
-        if 'posix' == os.name and os.path.expanduser('~') == '/root':
+            except ImportError:
+                logger.exception(_("Error loading the SQLite module: the antivirus software may be blocking it."))
+        if 'posix' == os.name and bleachbit.expanduser('~') == '/root':
             self.append_text(
                 _('You are running BleachBit with administrative privileges for cleaning shared parts of the system, and references to the user profile folder will clean only the root account.'))
+        if 'nt' == os.name and options.get('shred'):
+            from win32com.shell.shell import IsUserAnAdmin
+            if not IsUserAnAdmin():
+                self.append_text(
+                    _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
+                self.append_text('\n')
+        if exit:
+            # This is used for automated testing of whether the GUI can start.
+            gobject.idle_add(
+                lambda: gtk.main_quit(), priority=gobject.PRIORITY_LOW)
 
 
 if __name__ == '__main__':
