@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -21,15 +21,15 @@
 """
 Basic GUI code
 """
+from __future__ import absolute_import, print_function
 
+from bleachbit import _, expanduser
 
 import gtk
 import os
 
 if 'nt' == os.name:
-    import Windows
-
-from Common import _
+    from bleachbit import Windows
 
 
 def browse_folder(parent, title, multiple, stock_button):
@@ -48,7 +48,7 @@ def browse_folder(parent, title, multiple, stock_button):
                                         stock_button, gtk.RESPONSE_OK),
                                     action=gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
     chooser.set_select_multiple(multiple)
-    chooser.set_current_folder(os.path.expanduser('~'))
+    chooser.set_current_folder(expanduser('~'))
     resp = chooser.run()
     if multiple:
         ret = chooser.get_filenames()
@@ -72,7 +72,7 @@ def browse_file(parent, title):
                                     parent=parent,
                                     action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                     buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-    chooser.set_current_folder(os.path.expanduser('~'))
+    chooser.set_current_folder(expanduser('~'))
     resp = chooser.run()
     path = chooser.get_filename()
     chooser.destroy()
@@ -95,7 +95,7 @@ def browse_files(parent, title):
                                     action=gtk.FILE_CHOOSER_ACTION_OPEN,
                                     buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_DELETE, gtk.RESPONSE_OK))
     chooser.set_select_multiple(True)
-    chooser.set_current_folder(os.path.expanduser('~'))
+    chooser.set_current_folder(expanduser('~'))
     resp = chooser.run()
     paths = chooser.get_filenames()
     chooser.destroy()
@@ -165,7 +165,7 @@ def open_url(url, parent_window=None, prompt=True):
     if prompt:
         # find hostname
         import re
-        ret = re.search('^http(s)?://([a-z\.]+)', url)
+        ret = re.search('^http(s)?://([a-z.]+)', url)
         if None == ret:
             host = url
         else:
