@@ -2,8 +2,8 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2016 Andrew Ziem
-# http://www.bleachbit.org
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -23,6 +23,11 @@
 Integration specific to Unix-like operating systems
 """
 
+from __future__ import absolute_import, print_function
+
+import bleachbit
+from bleachbit import FileUtilities, General
+from bleachbit import _
 
 import glob
 import logging
@@ -30,19 +35,73 @@ import os
 import re
 import shlex
 import subprocess
-import ConfigParser
+import sys
 
-from Common import _, autostart_path
-import Common
-import FileUtilities
-import General
+logger = logging.getLogger(__name__)
+
+
+class LocaleCleanerPath:
+    """This represents a path with either a specific folder name or a folder name pattern.
+    It also may contain several compiled regex patterns for localization items (folders or files)
+    and additional LocaleCleanerPaths that get traversed when asked to supply a list of localization
+    items"""
+
+    def __init__(self, location):
+        if location is None:
+            raise RuntimeError("location is none")
+        self.pattern = location
+        self.children = []
+
+    def add_child(self, child):
+        """Adds a child LocaleCleanerPath"""
+        self.children.append(child)
+        return child
+
+    def add_path_filter(self, pre, post):
+        """Adds a filter consisting of a prefix and a postfix
+        (e.g. 'foobar_' and '\.qm' to match 'foobar_en_US.utf-8.qm)"""
+        try:
+            regex = re.compile('^' + pre + Locales.localepattern + post + '$')
+        except Exception as errormsg:
+            raise RuntimeError("Malformed regex '%s' or '%s': %s" % (pre, post, errormsg))
+        self.add_child(regex)
+
+    def get_subpaths(self, basepath):
+        """Returns direct subpaths for this object, i.e. either the named subfolder or all
+        subfolders matching the pattern"""
+        if isinstance(self.pattern, re._pattern_type):
+            return (os.path.join(basepath, p) for p in os.listdir(basepath)
+                    if self.pattern.match(p) and os.path.isdir(os.path.join(basepath, p)))
+        else:
+            path = os.path.join(basepath, self.pattern)
+            return [path] if os.path.isdir(path) else []
+
+    def get_localizations(self, basepath):
+        """Returns all localization items for this object and all descendant objects"""
+        for path in self.get_subpaths(basepath):
+            for child in self.children:
+                if isinstance(child, LocaleCleanerPath):
+                    for res in child.get_localizations(path):
+                        yield res
+                elif isinstance(child, re._pattern_type):
+                    for element in os.listdir(path):
+                        match = child.match(element)
+                        if match is not None:
+                            yield (match.group('locale'), os.path.join(path, element))
 
 
 class Locales:
-
     """Find languages and localization files"""
 
-    localepattern = r"(?P<locale>[a-z]{2,3})(?:[_-][a-zA-Z]{2,4})?(?:\.[a-zA-Z0-9-]*)?(?:@[a-zA-Z]*)?"
+    # The regular expression to match locale strings and extract the langcode.
+    # See test_locale_regex() in tests/TestUnix.py for examples
+    # This doesn't match all possible valid locale strings to avoid
+    # matching filenames you might want to keep, e.g. the regex
+    # to match jp.eucJP might also match jp.importantfileextension
+    localepattern =\
+        r'(?P<locale>[a-z]{2,3})' \
+        r'(?:[_-][A-Z]{2,4}(?:\.[\w]+[\d-]+|@\w+)?)?' \
+        r'(?P<encoding>[.-_](?:(?:ISO|iso|UTF|utf)[\d-]+|(?:euc|EUC)[A-Z]+))?'
 
     native_locale_names = \
         {'aa': 'Afaraf',
@@ -260,149 +319,74 @@ class Locales:
          'zh_TW': '中文',
          'zu': 'isiZulu'}
 
-    _paths = []
-
     def __init__(self):
-        pass
+        self._paths = LocaleCleanerPath(location='/')
 
-    def add_xml(self, xml_node):
-        self._paths.append(xml_node)
+    def add_xml(self, xml_node, parent=None):
+        """Parses the xml data and adds nodes to the LocaleCleanerPath-tree"""
+
+        if parent is None:
+            parent = self._paths
+        if xml_node.ELEMENT_NODE != xml_node.nodeType:
+            return
+
+        # if a pattern is supplied, we recurse into all matching subdirectories
+        if 'regexfilter' == xml_node.nodeName:
+            pre = xml_node.getAttribute('prefix') or ''
+            post = xml_node.getAttribute('postfix') or ''
+            parent.add_path_filter(pre, post)
+        elif 'path' == xml_node.nodeName:
+            if xml_node.hasAttribute('directoryregex'):
+                pattern = xml_node.getAttribute('directoryregex')
+                if '/' in pattern:
+                    raise RuntimeError('directoryregex may not contain slashes.')
+                pattern = re.compile(pattern)
+                parent = parent.add_child(LocaleCleanerPath(pattern))
+
+            # a combination of directoryregex and filter could be too much
+            else:
+                if xml_node.hasAttribute("location"):
+                    # if there's a filter attribute, it should apply to this path
+                    parent = parent.add_child(LocaleCleanerPath(xml_node.getAttribute('location')))
+
+                if xml_node.hasAttribute('filter'):
+                    userfilter = xml_node.getAttribute('filter')
+                    if 1 != userfilter.count('*'):
+                        raise RuntimeError(
+                            "Filter string '%s' must contain the placeholder * exactly once" % userfilter)
+
+                    # we can't use re.escape, because it escapes too much
+                    (pre, post) = (re.sub(r'([\[\]()^$.])', r'\\\1', p)
+                                   for p in userfilter.split('*'))
+                    parent.add_path_filter(pre, post)
+        else:
+            raise RuntimeError(
+                "Invalid node '%s', expected '<path>' or '<regexfilter>'" % xml_node.nodeName)
+
+        # handle child nodes
+        for child_xml in xml_node.childNodes:
+            self.add_xml(child_xml, parent)
 
     def localization_paths(self, locales_to_keep):
+        """Returns all localization items matching the previously added xml configuration"""
         if not locales_to_keep:
             raise RuntimeError('Found no locales to keep')
         purgeable_locales = frozenset((locale for locale in Locales.native_locale_names.keys()
                                        if locale not in locales_to_keep))
 
-        for xml_node in self._paths:
-            for (locale, path) in Locales.handle_path('', xml_node):
-                if locale in purgeable_locales:
-                    yield path
-
-    @staticmethod
-    def handle_path(path, xmldata):
-        """Extracts paths and filters from the supplied xml tree and yields matching files"""
-
-        if xmldata.ELEMENT_NODE != xmldata.nodeType:
-            return
-        if not xmldata.nodeName in ['path', 'regexfilter']:
-            raise RuntimeError(
-                "Invalid node '%s', expected '<path>' or '<regexfilter>'" % xmldata.nodeName)
-        location = xmldata.getAttribute('location') or '.'
-        if '.' != location:
-            path = path + location
-        if not path.endswith('/'):
-            path += '/'
-
-        if not os.path.isdir(path):
-            return
-
-        pre = None
-        post = None
-        if 'regexfilter' == xmldata.nodeName:
-            pre = xmldata.getAttribute('prefix') or ''
-            post = xmldata.getAttribute('postfix') or ''
-        if 'path' == xmldata.nodeName:
-            userfilter = xmldata.getAttribute('filter')
-            if not userfilter and not xmldata.hasChildNodes():
-                userfilter = '*'
-            if userfilter:
-                if 1 != userfilter.count('*'):
-                    raise RuntimeError(
-                        "Filter string '%s' must contain the placeholder * exactly once" % userfilter)
-
-                # we can't use re.escape, because it escapes too much
-                (pre, post) = (re.sub(r'([\[\]()^$.])', r'\\\1', p)
-                               for p in userfilter.split('*'))
-            # handle child nodes
-            for child in xmldata.childNodes:
-                for (locale, subpath) in Locales.handle_path(path, child):
-                    yield (locale, subpath)
-        if pre is not None and post is not None:
-            try:
-                re.compile(pre)
-                re.compile(post)
-            except Exception as errormsg:
-                raise RuntimeError(
-                    "Malformed regex '%s' or '%s': %s" % (pre, post, errormsg))
-            regex = re.compile('^' + pre + Locales.localepattern + post + '$')
-            for subpath in os.listdir(path):
-                match = regex.match(subpath)
-                if match is not None:
-                    yield (match.group("locale"), path + subpath)
-
-
-def apt_autoclean():
-    """Run 'apt-get autoclean' and return the size (un-rounded, in bytes)
-        of freed space"""
-
-    if not FileUtilities.exe_exists('apt-get'):
-        raise RuntimeError(_('Executable not found: %s') % 'apt-get')
-
-    args = ['apt-get', 'autoclean']
-
-    process = subprocess.Popen(args,
-                               stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
-
-    total_bytes = 0
-
-    while True:
-        line = process.stdout.readline().replace("\n", "")
-        if line.startswith('E: '):
-            raise RuntimeError(line)
-        # Del cups-common 1.3.9-17ubuntu3 [1165kB]
-        match = re.search("^Del .*\[([0-9.]+[a-zA-Z]{2})\]", line)
-        if match:
-            pkg_bytes_str = match.groups(0)[0]
-            pkg_bytes = FileUtilities.human_to_bytes(pkg_bytes_str.upper())
-            total_bytes += pkg_bytes
-        if "" == line and process.poll() != None:
-            break
-
-    return total_bytes
-
-
-def apt_autoremove():
-    """Run 'apt-get autoremove' and return the size (un-rounded, in bytes)
-        of freed space"""
-
-    if not FileUtilities.exe_exists('apt-get'):
-        raise RuntimeError(_('Executable not found: %s') % 'apt-get')
-
-    args = ['apt-get', '--yes', 'autoremove']
-
-    process = subprocess.Popen(args,
-                               stderr=subprocess.STDOUT, stdout=subprocess.PIPE)
-
-    total_bytes = 0
-
-    while True:
-        line = process.stdout.readline().replace("\n", "")
-        if line.startswith('E: '):
-            raise RuntimeError(line)
-        # After this operation, 74.7MB disk space will be freed.
-        match = re.search(
-            r", ([0-9.]+[a-zA-Z]{2}) disk space will be freed", line)
-        if match:
-            pkg_bytes_str = match.groups(0)[0]
-            pkg_bytes = FileUtilities.human_to_bytes(pkg_bytes_str.upper())
-            total_bytes += pkg_bytes
-        if "" == line and process.poll() != None:
-            break
-
-    return total_bytes
+        for (locale, path) in self._paths.get_localizations('/'):
+            if locale in purgeable_locales:
+                yield path
 
 
 def __is_broken_xdg_desktop_application(config, desktop_pathname):
     """Returns boolean whether application deskop entry file is broken"""
     if not config.has_option('Desktop Entry', 'Exec'):
-        print "info: is_broken_xdg_menu: missing required option 'Exec': '%s'" \
-            % (desktop_pathname)
+        logger.info("is_broken_xdg_menu: missing required option 'Exec': '%s'", desktop_pathname)
         return True
     exe = config.get('Desktop Entry', 'Exec').split(" ")[0]
     if not FileUtilities.exe_exists(exe):
-        print "info: is_broken_xdg_menu: executable '%s' does not exist '%s'" \
-            % (exe, desktop_pathname)
+        logger.info("is_broken_xdg_menu: executable '%s' does not exist '%s'", exe, desktop_pathname)
         return True
     if 'env' == exe:
         # Wine v1.0 creates .desktop files like this
@@ -420,16 +404,14 @@ def __is_broken_xdg_desktop_application(config, desktop_pathname):
             else:
                 break
         if not FileUtilities.exe_exists(execs[0]):
-            print "info: is_broken_xdg_menu: executable '%s'" \
-                "does not exist '%s'" % (execs[0], desktop_pathname)
+            logger.info("is_broken_xdg_menu: executable '%s' does not exist '%s'", execs[0], desktop_pathname)
             return True
         # check the Windows executable exists
         if wineprefix:
             windows_exe = wine_to_linux_path(wineprefix, execs[1])
             if not os.path.exists(windows_exe):
-                print "info: is_broken_xdg_menu: Windows executable" \
-                    "'%s' does not exist '%s'" % \
-                    (windows_exe, desktop_pathname)
+                logger.info("is_broken_xdg_menu: Windows executable '%s' does not exist '%s'",
+                            windows_exe, desktop_pathname)
                 return True
     return False
 
@@ -438,54 +420,62 @@ def is_unregistered_mime(mimetype):
     """Returns True if the MIME type is known to be unregistered. If
     registered or unknown, conservatively returns False."""
     try:
-        import gio
-        if 0 == len(gio.app_info_get_all_for_type(mimetype)):
+        from gi.repository import Gio
+        if 0 == len(Gio.app_info_get_all_for_type(mimetype)):
             return True
-    except:
-        logger = logging.getLogger(__name__)
-        logger.warning(
-            'error calling gio.app_info_get_all_for_type(%s)' % mimetype)
+    except ImportError:
+        logger.warning('error calling gio.app_info_get_all_for_type(%s)', mimetype)
     return False
 
 
 def is_broken_xdg_desktop(pathname):
     """Returns boolean whether the given XDG desktop entry file is broken.
     Reference: http://standards.freedesktop.org/desktop-entry-spec/latest/"""
-    config = ConfigParser.RawConfigParser()
+    config = bleachbit.RawConfigParser()
     config.read(pathname)
     if not config.has_section('Desktop Entry'):
-        print "info: is_broken_xdg_menu: missing required section " \
-            "'Desktop Entry': '%s'" % (pathname)
+        logger.info("is_broken_xdg_menu: missing required section 'Desktop Entry': '%s'", pathname)
         return True
     if not config.has_option('Desktop Entry', 'Type'):
-        print "info: is_broken_xdg_menu: missing required option 'Type': '%s'" % (pathname)
+        logger.info("is_broken_xdg_menu: missing required option 'Type': '%s'", pathname)
         return True
     file_type = config.get('Desktop Entry', 'Type').strip().lower()
     if 'link' == file_type:
         if not config.has_option('Desktop Entry', 'URL') and \
                 not config.has_option('Desktop Entry', 'URL[$e]'):
-            print "info: is_broken_xdg_menu: missing required option 'URL': '%s'" % (pathname)
+            logger.info("is_broken_xdg_menu: missing required option 'URL': '%s'", pathname)
             return True
         return False
     if 'mimetype' == file_type:
         if not config.has_option('Desktop Entry', 'MimeType'):
-            print "info: is_broken_xdg_menu: missing required option 'MimeType': '%s'" % (pathname)
+            logger.info("is_broken_xdg_menu: missing required option 'MimeType': '%s'", pathname)
             return True
         mimetype = config.get('Desktop Entry', 'MimeType').strip().lower()
         if is_unregistered_mime(mimetype):
-            print "info: is_broken_xdg_menu: MimeType '%s' not " \
-                "registered '%s'" % (mimetype, pathname)
+            logger.info("is_broken_xdg_menu: MimeType '%s' not registered '%s'", mimetype, pathname)
             return True
         return False
     if 'application' != file_type:
-        print "Warning: unhandled type '%s': file '%s'" % (file_type, pathname)
+        logger.warning("unhandled type '%s': file '%s'", file_type, pathname)
         return False
     if __is_broken_xdg_desktop_application(config, pathname):
         return True
     return False
 
 
-def is_running(exename):
+def is_running_darwin(exename, run_ps=None):
+    if run_ps is None:
+        def run_ps():
+            subprocess.check_output(["ps", "aux", "-c"])
+    try:
+        processess = (re.split(r"\s+", p, 10)[10] for p in run_ps().split("\n") if p != "")
+        next(processess)  # drop the header
+        return exename in processess
+    except IndexError:
+        raise RuntimeError("Unexpected output from ps")
+
+
+def is_running_linux(exename):
     """Check whether exename is running"""
     for filename in glob.iglob("/proc/*/exe"):
         try:
@@ -502,6 +492,16 @@ def is_running(exename):
     return False
 
 
+def is_running(exename):
+    """Check whether exename is running"""
+    if sys.platform.startswith('linux'):
+        return is_running_linux(exename)
+    elif 'darwin' == sys.platform:
+        return is_running_darwin(exename)
+    else:
+        raise RuntimeError('unsupported platform for physical_free()')
+
+
 def rotated_logs():
     """Yield a list of rotated (i.e., old) logs in /var/log/"""
     # Ubuntu 9.04
@@ -522,7 +522,7 @@ def rotated_logs():
     globpaths = ('/var/log/*-*', '/var/log/*/*-*')
     for path in FileUtilities.globex(globpaths, regex):
         whitelist_re = '^/var/log/(removed_)?(packages|scripts)'
-        if None == re.match(whitelist_re, path):  # for Slackware, Launchpad #367575
+        if re.match(whitelist_re, path) is None:  # for Slackware, Launchpad #367575
             yield path
 
 
@@ -531,28 +531,30 @@ def start_with_computer(enabled):
     If disabled, then delete the shortcut."""
     if not enabled:
         # User requests to not automatically start BleachBit
-        if os.path.lexists(autostart_path):
+        if os.path.lexists(bleachbit.autostart_path):
             # Delete the shortcut
-            FileUtilities.delete(autostart_path)
+            FileUtilities.delete(bleachbit.autostart_path)
         return
     # User requests to automatically start BleachBit
-    if os.path.lexists(autostart_path):
+    if os.path.lexists(bleachbit.autostart_path):
         # Already automatic, so exit
         return
-    if not os.path.exists(Common.launcher_path):
-        print 'ERROR: does not exist: ', Common.launcher_path
+    if not os.path.exists(bleachbit.launcher_path):
+        logger.error('%s does not exist: ', bleachbit.launcher_path)
         return
+    autostart_dir = os.path.dirname(bleachbit.autostart_path)
+    if not os.path.exists(autostart_dir):
+        General.makedirs(autostart_dir)
     import shutil
-    General.makedirs(os.path.dirname(autostart_path))
-    shutil.copy(Common.launcher_path, autostart_path)
-    os.chmod(autostart_path, 0755)
+    shutil.copy(bleachbit.launcher_path, bleachbit.autostart_path)
+    os.chmod(bleachbit.autostart_path, 0o755)
     if General.sudo_mode():
-        General.chownself(autostart_path)
+        General.chownself(bleachbit.autostart_path)
 
 
 def start_with_computer_check():
     """Return boolean whether BleachBit will start with the computer"""
-    return os.path.lexists(autostart_path)
+    return os.path.lexists(bleachbit.autostart_path)
 
 
 def wine_to_linux_path(wineprefix, windows_pathname):
@@ -564,37 +566,98 @@ def wine_to_linux_path(wineprefix, windows_pathname):
     return os.path.join(wineprefix, windows_pathname)
 
 
+def run_cleaner_cmd(cmd, args, freed_space_regex=r'[\d.]+[kMGTE]?B?', error_line_regexes=None):
+    """Runs a specified command and returns how much space was (reportedly) freed.
+    The subprocess shouldn't need any user input and the user should have the
+    necessary rights.
+    freed_space_regex gets applied to every output line, if the re matches,
+    add values captured by the single group in the regex"""
+    if not FileUtilities.exe_exists(cmd):
+        raise RuntimeError(_('Executable not found: %s') % cmd)
+    freed_space_regex = re.compile(freed_space_regex)
+    error_line_regexes = [re.compile(regex) for regex in error_line_regexes or []]
+
+    env = {'LC_ALL': 'C', 'PATH': os.getenv('PATH')}
+    output = subprocess.check_output([cmd] + args, stderr=subprocess.STDOUT,
+                                     universal_newlines=True, env=env)
+    freed_space = 0
+    for line in output.split('\n'):
+        m = freed_space_regex.match(line)
+        if m is not None:
+            freed_space += FileUtilities.human_to_bytes(m.group(1))
+        for error_re in error_line_regexes:
+            if error_re.search(line):
+                raise RuntimeError('Invalid output from %s: %s' % (cmd, line))
+
+    return freed_space
+
+
+def journald_clean():
+    """Clean the system journals"""
+    freed_space_regex = '^Vacuuming done, freed ([\d.]+[KMGT]?) of archived journals on disk.$'
+    return run_cleaner_cmd('journalctl', ['--vacuum-size=1'], freed_space_regex)
+
+
+def apt_autoremove():
+    """Run 'apt-get autoremove' and return the size (un-rounded, in bytes) of freed space"""
+
+    args = ['--yes', 'autoremove']
+    # After this operation, 74.7MB disk space will be freed.
+    freed_space_regex = r', ([\d.]+[a-zA-Z]{2}) disk space will be freed'
+    try:
+        return run_cleaner_cmd('apt-get', args, freed_space_regex, ['^E: '])
+    except subprocess.CalledProcessError as e:
+        raise RuntimeError("Error calling '%s':\n%s" % (' '.join(e.cmd), e.output))
+
+
+def apt_autoclean():
+    """Run 'apt-get autoclean' and return the size (un-rounded, in bytes) of freed space"""
+    try:
+        return run_cleaner_cmd('apt-get', ['autoclean'], r'^Del .*\[([\d.]+[a-zA-Z]{2})}]', ['^E: '])
+    except subprocess.CalledProcessError as e:
+        raise RuntimeError("Error calling '%s':\n%s" % (' '.join(e.cmd), e.output))
+
+
+def apt_clean():
+    """Run 'apt-get clean' and return the size in bytes of freed space"""
+    old_size = get_apt_size()
+    try:
+        run_cleaner_cmd('apt-get', ['clean'], '^unused regex$', ['^E: '])
+    except subprocess.CalledProcessError as e:
+        raise RuntimeError("Error calling '%s':\n%s" %
+                           (' '.join(e.cmd), e.output))
+    new_size = get_apt_size()
+    return old_size - new_size
+
+
+def get_apt_size():
+    """Return the size of the apt cache (in bytes)"""
+    (rc, stdout, stderr) = General.run_external(['apt-get', '-s', 'clean'])
+    paths = re.findall('/[/a-z\.\*]+', stdout)
+    return get_globs_size(paths)
+
+
+def get_globs_size(paths):
+    """Get the cumulative size (in bytes) of a list of globs"""
+    total_size = 0
+    for path in paths:
+        from glob import iglob
+        for p in iglob(path):
+            total_size += FileUtilities.getsize(p)
+    return total_size
+
+
 def yum_clean():
     """Run 'yum clean all' and return size in bytes recovered"""
     if os.path.exists('/var/run/yum.pid'):
         msg = _(
             "%s cannot be cleaned because it is currently running.  Close it, and try again.") % "Yum"
         raise RuntimeError(msg)
-    if not FileUtilities.exe_exists('yum'):
-        raise RuntimeError(_('Executable not found: %s') % 'yum')
+
     old_size = FileUtilities.getsizedir('/var/cache/yum')
-    args = ['yum', "--enablerepo=*", 'clean', 'all']
-    p = subprocess.Popen(args, stderr=subprocess.STDOUT,
-                         stdout=subprocess.PIPE)
-    non_blank_line = ""
-    while True:
-        line = p.stdout.readline().replace("\n", "")
-        if len(line) > 2:
-            non_blank_line = line
-        if -1 != line.find('You need to be root'):
-            # Seen before Fedora 13
-            raise RuntimeError(line)
-        if -1 != line.find('Cannot remove rpmdb file'):
-            # Since first in Fedora 13
-            raise RuntimeError(line)
-        if -1 != line.find('Another app is currently holding'):
-            print "debug: yum: '%s'" % line
-            old_size = FileUtilities.getsizedir('/var/cache/yum')
-        if "" == line and p.poll() != None:
-            break
-    print 'debug: yum process return code = %d' % p.returncode
-    if p.returncode > 0:
-        raise RuntimeError(non_blank_line)
+    args = ['--enablerepo=*', 'clean', 'all']
+    invalid = ['You need to be root', 'Cannot remove rpmdb file']
+    run_cleaner_cmd('yum', args, '^unused regex$', invalid)
     new_size = FileUtilities.getsizedir('/var/cache/yum')
     return old_size - new_size
 
