@@ -1,8 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2015 Andrew Ziem
-# http://bleachbit.sourceforge.net
+# Copyright (C) 2008-2018 Andrew Ziem
+# https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,14 +37,16 @@ These are the terms:
 
 """
 
+from __future__ import absolute_import, print_function
+
+import bleachbit
+from bleachbit import Command, FileUtilities, General
 
 import glob
 import logging
 import os
-import platform
 import re
 import sys
-import traceback
 
 from decimal import Decimal
 
@@ -63,10 +65,7 @@ if 'win32' == sys.platform:
     psapi = windll.psapi
     kernel = windll.kernel32
 
-import Command
-import Common
-import FileUtilities
-import General
+logger = logging.getLogger(__name__)
 
 
 def browse_file(_, title):
@@ -77,7 +76,7 @@ def browse_file(_, title):
                                         | win32con.OFN_FILEMUSTEXIST
                                         | win32con.OFN_HIDEREADONLY,
                                         Title=title)
-    except pywintypes.error, e:
+    except pywintypes.error as e:
         logger = logging.getLogger(__name__)
         if 0 == e.winerror:
             logger.debug('browse_file(): user cancelled')
@@ -96,8 +95,7 @@ def browse_files(_, title):
                                         | win32con.OFN_FILEMUSTEXIST
                                         | win32con.OFN_HIDEREADONLY,
                                         Title=title)
-    except pywintypes.error, e:
-        logger = logging.getLogger(__name__)
+    except pywintypes.error as e:
         if 0 == e.winerror:
             logger.debug('browse_files(): user cancelled')
         else:
@@ -117,7 +115,7 @@ def browse_files(_, title):
 def browse_folder(hwnd, title):
     """Ask the user to select a folder.  Return full path."""
     pidl = shell.SHBrowseForFolder(hwnd, None, title)[0]
-    if None == pidl:
+    if pidl is None:
         # user cancelled
         return None
     fullpath = shell.SHGetPathFromIDList(pidl)
@@ -126,12 +124,10 @@ def browse_folder(hwnd, title):
 
 def csidl_to_environ(varname, csidl):
     """Define an environment variable from a CSIDL for use in CleanerML and Winapp2.ini"""
-    logger = logging.getLogger(__name__)
     try:
         sppath = shell.SHGetSpecialFolderPath(None, csidl)
     except:
-        logger.exception(
-            'exception when getting special folder path for %s' % varname)
+        logger.info('exception when getting special folder path for %s', varname)
         return
     # there is exception handling in set_environ()
     set_environ(varname, sppath)
@@ -140,16 +136,10 @@ def csidl_to_environ(varname, csidl):
 def delete_locked_file(pathname):
     """Delete a file that is currently in use"""
     if os.path.exists(pathname):
-        try:
-            win32api.MoveFileEx(
-                pathname, None, win32con.MOVEFILE_DELAY_UNTIL_REBOOT)
-        except WindowsError, e:
-            if not 5 == e.winerror:
-                raise e
-            # show more useful message than "error: (5, 'MoveFileEx', 'Access
-            # is denied.')"
-            raise RuntimeError(
-                'Access denied when attempting to delete locked file %s' % pathname)
+        MOVEFILE_DELAY_UNTIL_REBOOT = 4
+        if 0 == windll.kernel32.MoveFileExW(pathname, None, MOVEFILE_DELAY_UNTIL_REBOOT):
+            from ctypes import WinError
+            raise WinError()
 
 
 def delete_registry_value(key, value_name, really_delete):
@@ -162,7 +152,7 @@ def delete_registry_value(key, value_name, really_delete):
         try:
             hkey = _winreg.OpenKey(hive, sub_key, 0, _winreg.KEY_SET_VALUE)
             _winreg.DeleteValue(hkey, value_name)
-        except WindowsError, e:
+        except WindowsError as e:
             if e.winerror == 2:
                 # 2 = 'file not found' means value does not exist
                 return False
@@ -172,7 +162,7 @@ def delete_registry_value(key, value_name, really_delete):
     try:
         hkey = _winreg.OpenKey(hive, sub_key)
         _winreg.QueryValueEx(hkey, value_name)
-    except WindowsError, e:
+    except WindowsError as e:
         if e.winerror == 2:
             return False
         raise
@@ -191,7 +181,7 @@ def delete_registry_key(parent_key, really_delete):
     hkey = None
     try:
         hkey = _winreg.OpenKey(hive, parent_sub_key)
-    except WindowsError, e:
+    except WindowsError as e:
         if e.winerror == 2:
             # 2 = 'file not found' happens when key does not exist
             return False
@@ -212,11 +202,11 @@ def delete_registry_key(parent_key, really_delete):
 
 def delete_updates():
     """Returns commands for deleting Windows Updates files"""
-    windir = os.path.expandvars('$windir')
+    windir = bleachbit.expandvars('$windir')
     dirs = glob.glob(os.path.join(windir, '$NtUninstallKB*'))
-    dirs += [os.path.expandvars('$windir\\SoftwareDistribution\\Download')]
-    dirs += [os.path.expandvars('$windir\\ie7updates')]
-    dirs += [os.path.expandvars('$windir\\ie8updates')]
+    dirs += [bleachbit.expandvars('$windir\\SoftwareDistribution\\Download')]
+    dirs += [bleachbit.expandvars('$windir\\ie7updates')]
+    dirs += [bleachbit.expandvars('$windir\\ie8updates')]
     if not dirs:
         # if nothing to delete, then also do not restart service
         return
@@ -250,7 +240,7 @@ def detect_registry_key(parent_key):
     hkey = None
     try:
         hkey = _winreg.OpenKey(hive, parent_sub_key)
-    except WindowsError, e:
+    except WindowsError as e:
         if e.winerror == 2:
             # 2 = 'file not found' happens when key does not exist
             return False
@@ -275,26 +265,23 @@ def elevate_privileges():
         # 10 = 10
         return False
 
-    logger = logging.getLogger(__name__)
-
     if shell.IsUserAnAdmin():
         logger.debug('already an admin (UAC not required)')
         return False
 
     if hasattr(sys, 'frozen'):
         # running frozen in py2exe
-        exe = unicode(sys.executable, sys.getfilesystemencoding())
+        exe = sys.executable.decode(sys.getfilesystemencoding())
         parameters = "--gui --no-uac"
     else:
         # __file__ is absolute path to bleachbit/Windows.py
-        pydir = os.path.dirname(unicode(__file__, sys.getfilesystemencoding()))
+        pydir = os.path.dirname(__file__.decode(sys.getfilesystemencoding()))
         pyfile = os.path.join(pydir, 'GUI.py')
         # If the Python file is on a network drive, do not offer the UAC because
         # the administrator may not have privileges and user will not be
         # prompted.
         if len(pyfile) > 0 and path_on_network(pyfile):
-            logger.debug(
-                "debug: skipping UAC because '%s' is on network" % pyfile)
+            logger.debug("debug: skipping UAC because '%s' is on network", pyfile)
             return False
         parameters = '"%s" --gui --no-uac' % pyfile
         exe = sys.executable
@@ -302,8 +289,7 @@ def elevate_privileges():
     # add any command line parameters such as --debug-log
     parameters = "%s %s" % (parameters, ' '.join(sys.argv[1:]))
 
-    logger.debug('elevate_privileges() exe=%s, parameters=%s' %
-                 (exe, parameters))
+    logger.debug('elevate_privileges() exe=%s, parameters=%s', exe, parameters)
 
     rc = None
     try:
@@ -311,13 +297,13 @@ def elevate_privileges():
                                   lpFile=exe,
                                   lpParameters=parameters,
                                   nShow=win32con.SW_SHOW)
-    except pywintypes.error, e:
+    except pywintypes.error as e:
         if 1223 == e.winerror:
             logger.debug('user denied the UAC dialog')
             return False
         raise
 
-    logger.debug('ShellExecuteEx=%s' % rc)
+    logger.debug('ShellExecuteEx=%s', rc)
 
     if isinstance(rc, dict):
         return True
@@ -349,23 +335,20 @@ def get_autostart_path():
         startupdir = shell.SHGetSpecialFolderPath(None, shellcon.CSIDL_STARTUP)
     except:
         # example of failure
-        # http://bleachbit.sourceforge.net/forum/error-windows-7-x64-bleachbit-091
-        logger = logging.getLogger(__name__)
+        # https://www.bleachbit.org/forum/error-windows-7-x64-bleachbit-091
         logger.exception('exception in get_autostart_path()')
         msg = 'Error finding user startup folder: %s ' % (
             str(sys.exc_info()[1]))
-        import GuiBasic
+        from bleachbit import GuiBasic
         GuiBasic.message_dialog(None, msg)
         # as a fallback, guess
         # Windows XP: C:\Documents and Settings\(username)\Start Menu\Programs\Startup
         # Windows 7:
         # C:\Users\(username)\AppData\Roaming\Microsoft\Windows\Start
         # Menu\Programs\Startup
-        startupdir = os.path.expandvars(
-            '$USERPROFILE\\Start Menu\\Programs\\Startup')
+        startupdir = bleachbit.expandvars('$USERPROFILE\\Start Menu\\Programs\\Startup')
         if not os.path.exists(startupdir):
-            startupdir = os.path.expandvars(
-                '$APPDATA\\Roaming\\Microsoft\\Windows\\Start Menu\\Programs\\Startup')
+            startupdir = bleachbit.expandvars('$APPDATA\\Roaming\\Microsoft\\Windows\\Start Menu\\Programs\\Startup')
     return os.path.join(startupdir, 'bleachbit.lnk')
 
 
@@ -378,7 +361,7 @@ def get_fixed_drives():
             # and free_space() returns access denied.
             # https://bugs.launchpad.net/bleachbit/+bug/1474848
             if os.path.isdir(drive):
-                yield drive
+                yield unicode(drive)
 
 
 def get_known_folder_path(folder_name):
@@ -468,11 +451,11 @@ def get_windows_version():
 def is_process_running(name):
     """Return boolean whether process (like firefox.exe) is running"""
 
-    if os.getenv('PROGRAMFILES(X86)'):
-        # 64-bit Windows detected
-        return is_process_running_wmic(name)
+    if parse_windows_build() >= 6:
+        return is_process_running_psutil(name)
     else:
-        # 32-bit Windows detected
+        # psutil does not support XP, so fall back
+        # https://github.com/giampaolo/psutil/issues/348
         return is_process_running_win32(name)
 
 
@@ -516,24 +499,28 @@ def is_process_running_win32(name):
                 # Filter out non-ASCII characters which we don't need
                 # and which may cause display warnings
                 clean_modname2 = re.sub(
-                    r'[^a-z\.]', '_', clean_modname.lower())
+                    r'[^a-z.]', '_', clean_modname.lower())
                 if clean_modname2 == name.lower():
                     return True
 
     return False
 
 
-def is_process_running_wmic(name):
+def is_process_running_psutil(name):
     """Return boolean whether process (like firefox.exe) is running
 
-    Works on Windows XP Professional but not on XP Home
+    Works on Windows Vista or later, but on Windows XP gives an ImportError
     """
 
-    clean_name = re.sub(r'[^A-Za-z\.]', '_', name).lower()
-    args = ['wmic', 'path', 'win32_process', 'where', "caption='%s'" %
-            clean_name, 'get', 'Caption']
-    (_, stdout, _) = General.run_external(args)
-    return stdout.lower().find(clean_name) > -1
+    import psutil
+    name = name.lower()
+    for proc in psutil.process_iter():
+        try:
+            if proc.name().lower() == name:
+                return True
+        except psutil.NoSuchProcess:
+            pass
+    return False
 
 
 def move_to_recycle_bin(path):
@@ -569,29 +556,23 @@ def shell_change_notify():
                          None, None)
     return 0
 
-
 def set_environ(varname, path):
     """Define an environment variable for use in CleanerML and Winapp2.ini"""
-    logger = logging.getLogger(__name__)
     if not path:
-        logger.debug('set_environ(%s, %s): skipping because blank path' %
-                     (varname, path))
-        # Such as LocalAppDataLow on XP
         return
-    if os.environ.has_key(varname):
-        logger.debug('set_environ(%s, %s): skipping because environment variable is already defined' %
-                     (varname, path))
+    if varname in os.environ:
+        #logger.debug('set_environ(%s, %s): skipping because environment variable is already defined', varname, path)
+        if 'nt' == os.name:
+            os.environ[varname] = bleachbit.expandvars(u'%%%s%%' % varname).encode('utf-8')
         # Do not redefine the environment variable when it already exists
+        # But re-encode them with utf-8 instead of mbcs
         return
     try:
         if not os.path.exists(path):
-            raise RuntimeError(
-                'Variable %s points to a non-existent path %s' % (varname, path))
-        os.environ[varname] = path
-        logger.debug('set_environ(%s, %s), set' % (varname, path))
+            raise RuntimeError('Variable %s points to a non-existent path %s' % (varname, path))
+        os.environ[varname] = path.encode('utf8')
     except:
-        logger.exception(
-            'set_environ(%s, %s): exception when setting environment variable' % (varname, path))
+        logger.exception('set_environ(%s, %s): exception when setting environment variable', varname, path)
 
 
 def setup_environment():
@@ -609,7 +590,6 @@ def setup_environment():
     try:
         path = get_known_folder_path('LocalAppDataLow')
     except:
-        logger = logging.getLogger(__name__)
         logger.exception('exception identifying LocalAppDataLow')
     else:
         set_environ('LocalAppDataLow', path)
@@ -627,7 +607,7 @@ def split_registry_key(full_key):
         'HKU': _winreg.HKEY_USERS}
     if k1 not in hive_map:
         raise RuntimeError("Invalid Windows registry hive '%s'" % k1)
-    return (hive_map[k1], k2)
+    return hive_map[k1], k2
 
 
 def start_with_computer(enabled):
@@ -640,12 +620,16 @@ def start_with_computer(enabled):
         return
     if os.path.lexists(autostart_path):
         return
-    import win32com.client
-    wscript_shell = win32com.client.Dispatch('WScript.Shell')
-    shortcut = wscript_shell.CreateShortCut(autostart_path)
-    shortcut.TargetPath = os.path.join(
-        Common.bleachbit_exe_path, 'bleachbit.exe')
-    shortcut.save()
+    import winshell
+    winshell.CreateShortcut(Path=autostart_path,
+                            Target=os.path.join(bleachbit.bleachbit_exe_path, 'bleachbit.exe'))
+
+    # import win32com.client
+    # wscript_shell = win32com.client.Dispatch('WScript.Shell')
+    # shortcut = wscript_shell.CreateShortCut(autostart_path)
+    # shortcut.TargetPath = os.path.join(
+    #     Common.bleachbit_exe_path, 'bleachbit.exe')
+    # shortcut.save()
 
 
 def start_with_computer_check():
